#include <SPI.h>
#include <RWM.h>

RWM rwm = RWM();

byte num_channels;                    // Number of channels to sample (can be 1, 2, or 4)
unsigned long samp_interval;          // Sampling interval in milliseconds (can range from 10 to 16,777,215)
int buffer[4];                        // Data buffer to store samples temporarily (size should match num_channels)

unsigned long address, time, num_lines;

void setup() {
  byte waiting;

  Serial.begin(115200);
  num_channels = rwm.EEPROMreadByte(0x1FFFC);
  if (num_channels==255) {
    Serial.println("EEPROM is blank.  You should log data before you read out the contents of the EEPROM.");
    while (1) {}
  }
  if ((num_channels!=1) && (num_channels!=2) && (num_channels!=4)) {
    Serial.print("The number of channels to log should be 1, 2, or 4, but ");
    Serial.print(num_channels);
    Serial.println(" were recorded in the EEPROM.");
    while (1) {}
  }
  samp_interval = (long(rwm.EEPROMreadByte(0x1FFFF))<<16)|long(word(rwm.EEPROMreadByte(0x1FFFE), rwm.EEPROMreadByte(0x1FFFD)));
  time = 0;
  address = 0;
  num_lines = 65534/num_channels;

  waiting = 1;
  while (waiting) {
    if (Serial.available()>0)
      waiting = (Serial.read()=='\r') ? 0:1;
  }
}

void loop() {
  if (address<num_lines) {
    Serial.print(time);
    Serial.print(",");
    if (num_channels==1)
      Serial.println(rwm.EEPROMreadInt(address));
    else if (num_channels==2) {
      rwm.EEPROMread2Ints(address, buffer);
      Serial.print(buffer[0]);
      Serial.print(",");
      Serial.println(buffer[1]);
    } else {
      rwm.EEPROMread4Ints(address, buffer);
      Serial.print(buffer[0]);
      Serial.print(",");
      Serial.print(buffer[1]);
      Serial.print(",");
      Serial.print(buffer[2]);
      Serial.print(",");
      Serial.println(buffer[3]);
    }
    time += samp_interval;
    ++address;
  } else {
    Serial.println();
    while (1) {}
  }
}
