#include <SPI.h>
#include <RWM.h>

RWM rwm = RWM();

byte num_channels = 2;                // Number of channels to sample (can be 1, 2, or 4)
unsigned long samp_interval = 10;     // Sampling interval in milliseconds (can range from 10 to 16,777,215)
int buffer[2];                        // Data buffer to store samples temporarily (size should match num_channels)

unsigned long address, next_samp_time;
int dac_val;

void setup() {
  byte value;

  Serial.begin(9600);
  value = rwm.EEPROMreadByte(0x1FFFC);
  if ((value!=255) && (value!=0)) {
    Serial.println("EEPROM is not blank.  You should download the data and clear the EEPROM before trying to log more data.");
    while (1) {}
  }
  if (value==255) {
    rwm.EEPROMwriteByte(0x1FFFC, 0);
    Serial.println("The datalogger will start logging data the next time it is powered up.");
    while (1) {}
  }
  if ((num_channels!=1) && (num_channels!=2) && (num_channels!=4)) {
    Serial.println("The number of channels to log must be 1, 2, or 4.");
    while (1) {}
  }
  if ((samp_interval<10) || (samp_interval>16777215)) {
    Serial.println("The sampling interval must be between 10ms and 16,777,215ms.");
    while (1) {}
  }
  rwm.EEPROMwriteByte(0x1FFFC, num_channels);
  rwm.EEPROMwriteByte(0x1FFFD, lowByte(word(samp_interval&0xFFFF)));
  rwm.EEPROMwriteByte(0x1FFFE, highByte(word(samp_interval&0xFFFF)));
  rwm.EEPROMwriteByte(0x1FFFF, byte(samp_interval>>16));
  address = 0;
  next_samp_time = millis() + samp_interval;
  dac_val = 0;
  rwm.DACwriteChannels(dac_val, 4000-dac_val);
}

void loop() {
  byte i;
  unsigned long time;

  time = millis();
  if (time>next_samp_time) {
    for (i = 0; i<num_channels; i++)
      buffer[i] = rwm.ADCreadChannel(i);
    rwm.EEPROMwrite2Ints(address, buffer);
    ++address;
    next_samp_time = time + samp_interval;
    ++dac_val;
    if (dac_val>4000)
      dac_val = 0;
    rwm.DACwriteChannels(dac_val, 4000-dac_val);
  }
}
